@@ -28,6 +28,7 @@
 #define   MRARTitle1         L"\nUsage:     rar <command> -<switch 1> -<switch N> <archive> <files...>"
 #define   MUNRARTitle1       L"\nUsage:     unrar <command> -<switch 1> -<switch N> <archive> <files...>"
 #define   MRARTitle2         L"\n               <@listfiles...> <path_to_extract\\>"
+#define   MFwrSlTitle2       L"\n               <@listfiles...> <path_to_extract/>"
 #define   MCHelpCmd          L"\n\n<Commands>"
 #define   MCHelpCmdA         L"\n  a             Add files to archive"
 #define   MCHelpCmdC         L"\n  c             Add archive comment"
@@ -58,6 +59,7 @@
 #define   MCHelpSwAD         L"\n  ad[1,2]       Alternate destination path"
 #define   MCHelpSwAG         L"\n  ag[format]    Generate archive name using the current date"
 #define   MCHelpSwAI         L"\n  ai            Ignore file attributes"
+#define   MCHelpSwAM         L"\n  am[s,r]       Archive name and time [save, restore]"
 #define   MCHelpSwAO         L"\n  ao            Add files with Archive attribute set"
 #define   MCHelpSwAP         L"\n  ap<path>      Set path inside archive"
 #define   MCHelpSwAS         L"\n  as            Synchronize archive contents"
@@ -92,9 +94,8 @@
 #define   MCHelpSwKB         L"\n  kb            Keep broken extracted files"
 #define   MCHelpSwLog        L"\n  log[f][=name] Write names to log file"
 #define   MCHelpSwMn         L"\n  m<0..5>       Set compression level (0-store...3-default...5-maximal)"
-#define   MCHelpSwMA         L"\n  ma[4|5]       Specify a version of archiving format"
 #define   MCHelpSwMC         L"\n  mc<par>       Set advanced compression parameters"
-#define   MCHelpSwMD         L"\n  md<n>[k,m,g]  Dictionary size in KB, MB or GB"
+#define   MCHelpSwMD         L"\n  md[x]<n>[kmg] Dictionary size in KB, MB or GB"
 #define   MCHelpSwME         L"\n  me[par]       Set encryption parameters"
 #define   MCHelpSwMS         L"\n  ms[ext;ext]   Specify file types to store"
 #define   MCHelpSwMT         L"\n  mt<threads>   Set the number of threads"
@@ -105,7 +106,7 @@
 #define   MCHelpSwOC         L"\n  oc            Set NTFS Compressed attribute"
 #define   MCHelpSwOH         L"\n  oh            Save hard links as the link instead of the file"
 #define   MCHelpSwOI         L"\n  oi[0-4][:min] Save identical files as references"
-#define   MCHelpSwOL         L"\n  ol[a]         Process symbolic links as the link [absolute paths]"
+#define   MCHelpSwOL         L"\n  ol[a,-]       Process symbolic links as the link [absolute paths, skip]"
 #define   MCHelpSwONI        L"\n  oni           Allow potentially incompatible names"
 #define   MCHelpSwOP         L"\n  op<path>      Set the output path for extracted files"
 #define   MCHelpSwOR         L"\n  or            Rename files automatically"
@@ -124,8 +125,8 @@
 #define   MCHelpSwSC         L"\n  sc<chr>[obj]  Specify the character set"
 #define   MCHelpSwSFX        L"\n  sfx[name]     Create SFX archive"
 #define   MCHelpSwSI         L"\n  si[name]      Read data from standard input (stdin)"
-#define   MCHelpSwSL         L"\n  sl<size>      Process files with size less than specified"
-#define   MCHelpSwSM         L"\n  sm<size>      Process files with size more than specified"
+#define   MCHelpSwSL         L"\n  sl<size>[u]   Process files with size less than specified"
+#define   MCHelpSwSM         L"\n  sm<size>[u]   Process files with size more than specified"
 #define   MCHelpSwT          L"\n  t             Test files after archiving"
 #define   MCHelpSwTK         L"\n  tk            Keep original archive time"
 #define   MCHelpSwTL         L"\n  tl            Set archive time to latest file"
@@ -137,10 +138,9 @@
 #define   MCHelpSwU          L"\n  u             Update files"
 #define   MCHelpSwV          L"\n  v             Create volumes with size autodetection or list all volumes"
 #define   MCHelpSwVUnr       L"\n  v             List all volumes"
-#define   MCHelpSwVn         L"\n  v<size>[k,b]  Create volumes with size=<size>*1000 [*1024, *1]"
+#define   MCHelpSwVn         L"\n  v<size>[u]    Create volumes with size in [bBkKmMgGtT] units"
 #define   MCHelpSwVD         L"\n  vd            Erase disk contents before creating volume"
 #define   MCHelpSwVER        L"\n  ver[n]        File version control"
-#define   MCHelpSwVN         L"\n  vn            Use the old style volume naming scheme"
 #define   MCHelpSwVP         L"\n  vp            Pause before each volume"
 #define   MCHelpSwW          L"\n  w<path>       Assign work directory"
 #define   MCHelpSwX          L"\n  x<file>       Exclude specified file"
@@ -229,12 +229,11 @@
 #define   MMainHeaderBroken  L"\nMain archive header is corrupt"
 #define   MLogFileHead       L"\n%s - the file header is corrupt"
 #define   MLogProtectHead    L"The data recovery header is corrupt"
+#define   MArcComment        L"\nArchive comment"
 #define   MReadStdinCmt      L"\nReading comment from stdin\n"
 #define   MReadCommFrom      L"\nReading comment from %s"
 #define   MDelComment        L"\nDeleting comment from %s"
 #define   MAddComment        L"\nAdding comment to %s"
-#define   MFCommAdd          L"\nAdding file comments"
-#define   MAskFComm          L"\n\nReading comment for %s : %s from stdin\n"
 #define   MLogCommBrk        L"\nThe archive comment is corrupt"
 #define   MCommAskCont       L"\nPress 'Enter' to continue or 'Q' to quit:"
 #define   MWriteCommTo       L"\nWrite comment to %s"
@@ -282,11 +281,9 @@
 #define   MListHostOS        L"Host OS"
 #define   MListFileVer       L"File version"
 #define   MListService       L"Service"
-#define   MListUOHead        L"\n   Unix Owner/Group data:    %-14s %-14s"
 #define   MListNTACLHead     L"\n   NTFS security data"
 #define   MListStrmHead      L"\n   NTFS stream: %s"
 #define   MListUnkHead       L"\n   Unknown subheader type: 0x%04x"
-#define   MFileComment       L"\nComment: "
 #define   MYes               L"Yes"
 #define   MNo                L"No"
 #define   MListNoFiles       L"  0 files\n"
@@ -294,7 +291,7 @@
 #define   MRprBuild          L"\nBuilding %s"
 #define   MRprOldFormat      L"\nCannot repair archive with old format"
 #define   MRprFind           L"\nFound  %s"
-#define   MRprAskIsSol       L"\nThe archive header is corrupt. Mark archive as solid ?"
+#define   MRprAskIsSol       L"\nThe archive header is corrupt. Mark archive as solid?"
 #define   MRprNoFiles        L"\nNo files found"
 #define   MLogUnexpEOF       L"\nUnexpected end of archive"
 #define   MRepAskReconst     L"\nReconstruct archive structure ?"
@@ -328,6 +325,7 @@
 #define   MCorrectingName    L"\nWARNING: Attempting to correct the invalid file or directory name"
 #define   MUnpCannotMerge    L"\nWARNING: You need to start extraction from a previous volume to unpack %s"
 #define   MUnknownOption     L"\nERROR: Unknown option: %s"
+#define   MSwSyntaxError     L"\nERROR: '-' is expected in the beginning of: %s"
 #define   MSubHeadCorrupt    L"\nERROR: Corrupt data header found, ignored"
 #define   MSubHeadUnknown    L"\nWARNING: Unknown data header format, ignored"
 #define   MSubHeadDataCRC    L"\nERROR: Corrupt %s data block"
@@ -384,6 +382,8 @@
 #define   MNeedAdmin         L"\nYou may need to run RAR as administrator"
 #define   MDictOutMem        L"\nNot enough memory for %d MB compression dictionary, changed to %d MB."
 #define   MUseSmalllerDict   L"\nPlease use a smaller compression dictionary."
+#define   MExtrDictOutMem    L"\nNot enough memory to unpack the archive with %u GB compression dictionary."
+#define   MSuggest64bit      L"\n64-bit RAR version is necessary."
 #define   MOpenErrAtime      L"\nYou may need to remove -tsp switch to open this file."
 #define   MErrReadInfo       L"\nChoose 'Ignore' to continue with the already read file part only, 'Ignore all' to do it for all read errors, 'Retry' to repeat read and 'Quit' to abort."
 #define   MErrReadTrunc      L"\n%s is archived incompletely because of read error.\n"
@@ -394,3 +394,10 @@
 #define   MAdjustValue       L"\nAdjusting %s value to %s."
 #define   MOpFailed          L"\nOperation failed"
 #define   MSkipEncArc        L"\nSkipping the encrypted archive %s"
+#define   MOrigName          L"Original name"
+#define   MOriginalTime      L"Original time"
+#define   MFileRenamed       L"\n%s is renamed to %s"
+#define   MDictNotAllowed    L"\n%u GB dictionary exceeds %u GB limit and needs more than %u GB memory to unpack."
+#define   MDictExtrAnyway    L"\nUse -md%ug or -mdx%ug switches to extract anyway."
+#define   MDictComprLimit    L"\n%u GB dictionary exceeds %u GB limit and not allowed when compressing data."
+#define   MNeedSFX64         L"\n64-bit self-extracting module is necessary for %u GB compression dictionary."
