@@ -1,12 +1,11 @@
 static bool AnyMessageDisplayed=false; // For console -idn switch.
 
 // Purely user interface function. Gets and returns user input.
-UIASKREP_RESULT uiAskReplace(wchar *Name,size_t MaxNameSize,int64 FileSize,RarTime *FileTime,uint Flags)
+UIASKREP_RESULT uiAskReplace(std::wstring &Name,int64 FileSize,RarTime *FileTime,uint Flags)
 {
   wchar SizeText1[20],DateStr1[50],SizeText2[20],DateStr2[50];
 
-  FindData ExistingFD;
-  memset(&ExistingFD,0,sizeof(ExistingFD)); // In case find fails.
+  FindData ExistingFD={}; // Init in case find fails.
   FindFile::FastFind(Name,&ExistingFD);
   itoa(ExistingFD.Size,SizeText1,ASIZE(SizeText1));
   ExistingFD.mtime.GetText(DateStr1,ASIZE(DateStr1),false);
@@ -14,16 +13,16 @@ UIASKREP_RESULT uiAskReplace(wchar *Name,size_t MaxNameSize,int64 FileSize,RarTi
   if (FileSize==INT64NDF || FileTime==NULL)
   {
     eprintf(L"\n");
-    eprintf(St(MAskOverwrite),Name);
+    eprintf(St(MAskOverwrite),Name.c_str());
   }
   else
   {
     itoa(FileSize,SizeText2,ASIZE(SizeText2));
     FileTime->GetText(DateStr2,ASIZE(DateStr2),false);
     if ((Flags & UIASKREP_F_EXCHSRCDEST)==0)
-      eprintf(St(MAskReplace),Name,SizeText1,DateStr1,SizeText2,DateStr2);
+      eprintf(St(MAskReplace),Name.c_str(),SizeText1,DateStr1,SizeText2,DateStr2);
     else
-      eprintf(St(MAskReplace),Name,SizeText2,DateStr2,SizeText1,DateStr1);
+      eprintf(St(MAskReplace),Name.c_str(),SizeText2,DateStr2,SizeText1,DateStr1);
   }
 
   bool AllowRename=(Flags & UIASKREP_F_NORENAME)==0;
@@ -46,7 +45,7 @@ UIASKREP_RESULT uiAskReplace(wchar *Name,size_t MaxNameSize,int64 FileSize,RarTi
   if (AllowRename && Choice==5)
   {
     mprintf(St(MAskNewName));
-    if (getwstr(Name,MaxNameSize))
+    if (getwstr(Name))
       return UIASKREP_R_RENAME;
     else
       return UIASKREP_R_SKIP; // Process fwgets failure as if user answered 'No'.
@@ -57,13 +56,13 @@ UIASKREP_RESULT uiAskReplace(wchar *Name,size_t MaxNameSize,int64 FileSize,RarTi
 
 
 
-void uiStartArchiveExtract(bool Extract,const wchar *ArcName)
+void uiStartArchiveExtract(bool Extract,const std::wstring &ArcName)
 {
-  mprintf(St(Extract ? MExtracting : MExtrTest), ArcName);
+  mprintf(St(Extract ? MExtracting : MExtrTest), ArcName.c_str());
 }
 
 
-bool uiStartFileExtract(const wchar *FileName,bool Extract,bool Test,bool Skip)
+bool uiStartFileExtract(const std::wstring &FileName,bool Extract,bool Test,bool Skip)
 {
   return true;
 }
@@ -183,6 +182,7 @@ void uiMsgStore::Msg()
       Log(NULL,St(MNeedAdmin));
       break;
     case UIERROR_ARCBROKEN:
+      mprintf(L"\n"); // So it is not merged with preceding UIERROR_HEADERBROKEN.
       Log(Str[0],St(MErrBrokenArc));
       break;
     case UIERROR_HEADERBROKEN:
@@ -236,6 +236,12 @@ void uiMsgStore::Msg()
     case UIERROR_RECVOLCANNOTFIX:
       mprintf(St(MRecVolCannotFix));
       break;
+    case UIERROR_EXTRDICTOUTMEM:
+      Log(Str[0],St(MExtrDictOutMem),Num[0]);
+#ifdef _WIN_32
+      Log(Str[0],St(MSuggest64bit));
+#endif
+      break;
     case UIERROR_UNEXPEOF:
       Log(Str[0],St(MLogUnexpEOF));
       break;
@@ -388,7 +394,6 @@ void uiMsgStore::Msg()
       break;
 
 
-
     case UIEVENT_RRTESTINGSTART:
       mprintf(L"%s      ",St(MTestingRR));
       break;
@@ -396,7 +401,7 @@ void uiMsgStore::Msg()
 }
 
 
-bool uiGetPassword(UIPASSWORD_TYPE Type,const wchar *FileName,
+bool uiGetPassword(UIPASSWORD_TYPE Type,const std::wstring &FileName,
                    SecPassword *Password,CheckPassword *CheckPwd)
 {
   // Unlike GUI we cannot provide Cancel button here, so we use the empty
@@ -432,14 +437,14 @@ void uiAlarm(UIALARM_TYPE Type)
 
 
 
-bool uiAskNextVolume(wchar *VolName,size_t MaxSize)
+bool uiAskNextVolume(std::wstring &VolName)
 {
-  eprintf(St(MAskNextVol),VolName);
+  eprintf(St(MAskNextVol),VolName.c_str());
   return Ask(St(MContinueQuit))!=2;
 }
 
 
-void uiAskRepeatRead(const wchar *FileName,bool &Ignore,bool &All,bool &Retry,bool &Quit)
+void uiAskRepeatRead(const std::wstring &FileName,bool &Ignore,bool &All,bool &Retry,bool &Quit)
 {
   eprintf(St(MErrReadInfo));
   int Code=Ask(St(MIgnoreAllRetryQuit));
@@ -451,16 +456,37 @@ void uiAskRepeatRead(const wchar *FileName,bool &Ignore,bool &All,bool &Retry,bo
 }
 
 
-bool uiAskRepeatWrite(const wchar *FileName,bool DiskFull)
+bool uiAskRepeatWrite(const std::wstring &FileName,bool DiskFull)
 {
   mprintf(L"\n");
-  Log(NULL,St(DiskFull ? MNotEnoughDisk:MErrWrite),FileName);
+  Log(NULL,St(DiskFull ? MNotEnoughDisk:MErrWrite),FileName.c_str());
   return Ask(St(MRetryAbort))==1;
 }
 
 
+bool uiDictLimit(CommandData *Cmd,const std::wstring &FileName,uint64 DictSize,uint64 MaxDictSize)
+{
+  mprintf(L"\n%s",FileName.c_str());
+  const uint64 GB=1024*1024*1024; // We display sizes in GB here.
+
+  // Include the reminder for switches like -md6400m.
+  DictSize=DictSize/GB+(DictSize%GB!=0 ? 1:0);
+
+  // Exclude reminder for in case it is less than archive dictionary size,
+  // but still more than nearest GB value. Otherwise we could have message
+  // like "7 GB dictionary exceeds 7 GB limit", where first 7 might be actually
+  // 6272 MB and second is 6400 MB.
+  MaxDictSize/=GB;
+
+  mprintf(St(MDictNotAllowed),(uint)DictSize,(uint)MaxDictSize,(uint)DictSize);
+  mprintf(St(MDictExtrAnyway),(uint)DictSize,(uint)DictSize);
+  mprintf(L"\n");
+  return false; // Stop extracting.
+}
+
+
 #ifndef SFX_MODULE
-const wchar *uiGetMonthName(int Month)
+const wchar *uiGetMonthName(uint Month)
 {
   static MSGID MonthID[12]={
          MMonthJan,MMonthFeb,MMonthMar,MMonthApr,MMonthMay,MMonthJun,
