@@ -128,6 +128,26 @@ bool CharToWide(const char *Src,wchar *Dest,size_t DestSize)
 }
 
 
+bool WideToChar(const std::wstring &Src,std::string &Dest)
+{
+  // We need more than 1 char per wchar_t for DBCS and up to 4 for UTF-8.
+  std::vector<char> DestA(4*Src.size()+1); // "+1" for terminating zero.
+  bool Result=WideToChar(Src.c_str(),DestA.data(),DestA.size());
+  Dest=DestA.data();
+  return Result;
+}
+
+
+bool CharToWide(const std::string &Src,std::wstring &Dest)
+{
+  // 2 wchar_t per char in case char is converted to UTF-16 surrogate pair.
+  std::vector<wchar> DestW(2*Src.size()+1); // "+1" for terminating zero.
+  bool Result=CharToWide(Src.c_str(),DestW.data(),DestW.size());
+  Dest=DestW.data();
+  return Result;
+}
+
+
 #if defined(_UNIX) && defined(MBFUNCTIONS)
 // Convert and restore mapped inconvertible Unicode characters. 
 // We use it for extended ASCII names in Unix.
@@ -229,10 +249,11 @@ void CharToWideMap(const char *Src,wchar *Dest,size_t DestSize,bool &Success)
 #endif
 
 
-// SrcSize is in wide characters, not in bytes.
-byte* WideToRaw(const wchar *Src,byte *Dest,size_t SrcSize)
+// SrcSize is source data size in wide characters, not in bytes.
+// DestSize is the maximum allowed destination size.
+byte* WideToRaw(const wchar *Src,size_t SrcSize,byte *Dest,size_t DestSize)
 {
-  for (size_t I=0;I<SrcSize;I++,Src++)
+  for (size_t I=0;I<SrcSize && I*2+1<DestSize;I++,Src++)
   {
     Dest[I*2]=(byte)*Src;
     Dest[I*2+1]=(byte)(*Src>>8);
@@ -243,6 +264,19 @@ byte* WideToRaw(const wchar *Src,byte *Dest,size_t SrcSize)
 }
 
 
+// Store UTF-16 raw byte stream.
+void WideToRaw(const std::wstring &Src,std::vector<byte> &Dest)
+{
+  for (wchar C : Src)
+  {
+    Dest.push_back((byte)C);
+    Dest.push_back((byte)(C>>8));
+  }
+  Dest.push_back(0); // 2 bytes of trailing UTF-16 zero.
+  Dest.push_back(0);
+}
+
+
 wchar* RawToWide(const byte *Src,wchar *Dest,size_t DestSize)
 {
   for (size_t I=0;I<DestSize;I++)
@@ -252,6 +286,20 @@ wchar* RawToWide(const byte *Src,wchar *Dest,size_t DestSize)
 }
 
 
+std::wstring RawToWide(const std::vector<byte> &Src)
+{
+  std::wstring Dest;
+  for (size_t I=0;I+1<Src.size();I+=2)
+  {
+    wchar c=Src[I]+(Src[I+1]<<8);
+    Dest.push_back(c);
+    if (c==0)
+      break;
+  }
+  return Dest;
+}
+
+
 void WideToUtf(const wchar *Src,char *Dest,size_t DestSize)
 {
   long dsize=(long)DestSize;
@@ -294,6 +342,46 @@ void WideToUtf(const wchar *Src,char *Dest,size_t DestSize)
 }
 
 
+void WideToUtf(const std::wstring &Src,std::string &Dest)
+{
+  for (size_t I=0;I<Src.size() && Src[I]!=0;)
+  {
+    uint c=Src[I++];
+    if (c<0x80)
+      Dest.push_back(c);
+    else
+      if (c<0x800)
+      {
+        Dest.push_back(0xc0|(c>>6));
+        Dest.push_back(0x80|(c&0x3f));
+      }
+      else
+      {
+        if (c>=0xd800 && c<=0xdbff && I<Src.size() && Src[I]>=0xdc00 && Src[I]<=0xdfff) // Surrogate pair.
+        {
+          c=((c-0xd800)<<10)+(Src[I]-0xdc00)+0x10000;
+          I++;
+        }
+        if (c<0x10000)
+        {
+          Dest.push_back(0xe0|(c>>12));
+          Dest.push_back(0x80|((c>>6)&0x3f));
+          Dest.push_back(0x80|(c&0x3f));
+        }
+        else
+          if (c < 0x200000)
+          {
+            Dest.push_back(0xf0|(c>>18));
+            Dest.push_back(0x80|((c>>12)&0x3f));
+            Dest.push_back(0x80|((c>>6)&0x3f));
+            Dest.push_back(0x80|(c&0x3f));
+          }
+      }
+  }
+}
+
+
+
 size_t WideToUtfSize(const wchar *Src)
 {
   size_t Size=0;
@@ -396,6 +484,146 @@ bool UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
 }
 
 
+bool UtfToWide(const char *Src,std::wstring &Dest)
+{
+  bool Success=true;
+  Dest.clear();
+  while (*Src!=0)
+  {
+    uint c=byte(*(Src++)),d;
+    if (c<0x80)
+      d=c;
+    else
+      if ((c>>5)==6)
+      {
+        if ((*Src&0xc0)!=0x80)
+        {
+          Success=false;
+          break;
+        }
+        d=((c&0x1f)<<6)|(*Src&0x3f);
+        Src++;
+      }
+      else
+        if ((c>>4)==14)
+        {
+          if ((Src[0]&0xc0)!=0x80 || (Src[1]&0xc0)!=0x80)
+          {
+            Success=false;
+            break;
+          }
+          d=((c&0xf)<<12)|((Src[0]&0x3f)<<6)|(Src[1]&0x3f);
+          Src+=2;
+        }
+        else
+          if ((c>>3)==30)
+          {
+            if ((Src[0]&0xc0)!=0x80 || (Src[1]&0xc0)!=0x80 || (Src[2]&0xc0)!=0x80)
+            {
+              Success=false;
+              break;
+            }
+            d=((c&7)<<18)|((Src[0]&0x3f)<<12)|((Src[1]&0x3f)<<6)|(Src[2]&0x3f);
+            Src+=3;
+          }
+          else
+          {
+            Success=false;
+            break;
+          }
+    if (d>0xffff)
+    {
+      if (d>0x10ffff) // UTF-8 must end at 0x10ffff according to RFC 3629.
+      {
+        Success=false;
+        continue;
+      }
+      if (sizeof(wchar_t)==2) // Use the surrogate pair.
+      {
+        Dest.push_back( ((d-0x10000)>>10)+0xd800 );
+        Dest.push_back( (d&0x3ff)+0xdc00 );
+      }
+      else
+        Dest.push_back( d );
+    }
+    else
+      Dest.push_back( d );
+  }
+  return Success;
+}
+
+
+/*
+bool UtfToWide(const std::vector<char> &Src,std::wstring &Dest)
+{
+  bool Success=true;
+  Dest.clear();
+  for (size_t I=0;I<Src.size() && Src[I]!=0;) // We expect it to always stop at 0.
+  {
+    uint c=byte(Src[I++]),d;
+    if (c<0x80)
+      d=c;
+    else
+      if ((c>>5)==6)
+      {
+        if (Src.size()-I<1 || (Src[I]&0xc0)!=0x80)
+        {
+          Success=false;
+          break;
+        }
+        d=((c&0x1f)<<6)|(Src[I]&0x3f);
+        I++;
+      }
+      else
+        if ((c>>4)==14)
+        {
+          if (Src.size()-I<2 || (Src[I]&0xc0)!=0x80 || (Src[I+1]&0xc0)!=0x80)
+          {
+            Success=false;
+            break;
+          }
+          d=((c&0xf)<<12)|((Src[I]&0x3f)<<6)|(Src[I+1]&0x3f);
+          I+=2;
+        }
+        else
+          if ((c>>3)==30)
+          {
+            if (Src.size()-I<3 || (Src[I]&0xc0)!=0x80 || (Src[I+1]&0xc0)!=0x80 || (Src[I+2]&0xc0)!=0x80)
+            {
+              Success=false;
+              break;
+            }
+            d=((c&7)<<18)|((Src[I]&0x3f)<<12)|((Src[I+1]&0x3f)<<6)|(Src[I+2]&0x3f);
+            I+=3;
+          }
+          else
+          {
+            Success=false;
+            break;
+          }
+    if (d>0xffff)
+    {
+      if (d>0x10ffff) // UTF-8 must end at 0x10ffff according to RFC 3629.
+      {
+        Success=false;
+        continue;
+      }
+      if (sizeof(Dest[0])==2) // Use the surrogate pair.
+      {
+        Dest.push_back( ((d-0x10000)>>10)+0xd800 );
+        Dest.push_back( (d&0x3ff)+0xdc00 );
+      }
+      else
+        Dest.push_back( d );
+    }
+    else
+      Dest.push_back( d );
+  }
+  return Success;
+}
+*/
+
+
 // For zero terminated strings.
 bool IsTextUtf8(const byte *Src)
 {
@@ -482,7 +710,15 @@ const wchar_t* wcscasestr(const wchar_t *str, const wchar_t *search)
       if (tolowerw(str[i+j])!=tolowerw(search[j]))
         break;
     }
-  return NULL;
+  return nullptr;
+}
+
+
+// Case insensitive std::wstring substring search.
+std::wstring::size_type wcscasestr(const std::wstring &str, const std::wstring &search)
+{
+  const wchar *Found=wcscasestr(str.c_str(),search.c_str());
+  return Found==nullptr ? std::wstring::npos : Found-str.c_str();
 }
 
 
@@ -499,10 +735,14 @@ wchar* wcslower(wchar *s)
 #endif
   return s;
 }
-#endif
 
 
-#ifndef SFX_MODULE
+void wcslower(std::wstring &s)
+{
+  wcslower(&s[0]);
+}
+
+
 wchar* wcsupper(wchar *s)
 {
 #ifdef _WIN_ALL
@@ -515,6 +755,12 @@ wchar* wcsupper(wchar *s)
 #endif
   return s;
 }
+
+
+void wcsupper(std::wstring &s)
+{
+  wcsupper(&s[0]);
+}
 #endif
 
 
@@ -547,27 +793,28 @@ int tolowerw(int ch)
 }
 
 
-int atoiw(const wchar *s)
+int atoiw(const std::wstring &s)
 {
   return (int)atoilw(s);
 }
 
 
-int64 atoilw(const wchar *s)
+int64 atoilw(const std::wstring &s)
 {
   bool sign=false;
-  if (*s=='-') // We do use signed integers here, for example, in GUI SFX.
+  size_t Pos=0;
+  if (s[Pos]=='-') // We do use signed integers here, for example, in GUI SFX.
   {
-    s++;
+    Pos++;
     sign=true;
   }
   // Use unsigned type here, since long string can overflow the variable
   // and signed integer overflow is undefined behavior in C++.
   uint64 n=0;
-  while (*s>='0' && *s<='9')
+  while (s[Pos]>='0' && s[Pos]<='9')
   {
-    n=n*10+(*s-'0');
-    s++;
+    n=n*10+(s[Pos]-'0');
+    Pos++;
   }
   // Check int64(n)>=0 to avoid the signed overflow with undefined behavior
   // when negating 0x8000000000000000.
